
# requires either Python 2.6 or the odict module
try:
    from odict import OrderedDict
except ImportError:
    from xamlwriter.modules.odict import OrderedDict

    
not_implemented = lambda *_: NotImplemented


class Node(object):
    def __init__(self, name):
        self.name = name
        self.attributes = OrderedDict()
        self.children = []
        self.parent = None


    def __eq__(self, other):
        return (type(other) == type(self) and
                self.name == other.name and
                dict(self.attributes) == dict(other.attributes) and
                self.children == other.children)


    def __ne__(self, other):
        return not self == other
    
    __gt__ = __lt__ = __ge__ = __le__ = not_implemented
    
    __hash__ = None


    def to_string(self):
        attribute_string = "".join(' %s="%s"' % (key, value) for key, value in 
                                    self.attributes.items())
        name = self.name
        if not self.children:
            return '<%s />' % (name + attribute_string)
        children_string = "".join(node.to_string() for node in self.children)
        return '<%s>%s</%s>' % (name + attribute_string, children_string, self.name)

    # Hmmm...
    __repr__ = to_string


class TextNode(object):
    
    def __init__(self, data):
        self.data = data
    
    def __eq__(self, other):
        return (type(other) == type(self) and
                self.data == other.data)
    
    def __ne__(self, other):
        return not self == other
    
    __gt__ = __lt__ = __ge__ = __le__ = not_implemented
    
    __hash__ = None


    def to_string(self):
        return self.data

    # Hmmm...
    __repr__ = to_string



class ErrorNode(Node):
    pass




from docutils.core import publish_string
from docutils.writers import Writer
from xamlwriter.translator import XamlTranslator


class XamlWriter(Writer):
    """Writer to convert a docutils nodetree to a XAML nodetree."""
    
    def __init__(self, flowdocument=True, xclass=True):
        self.flowdocument = flowdocument
        self.xclass = xclass
        Writer.__init__(self)

    supported = ('xaml',)
    output = None

    def translate(self):
        visitor = XamlTranslator(self.document, flowdocument=self.flowdocument, xclass=self.xclass)
        self.document.walkabout(visitor)
        self.output = visitor

settings_overrides = {
    'file_insertion_enabled': False,
}

def publish_xaml(input_data, flowdocument=True, overrides=None, xclass=True):
    config = settings_overrides.copy()
    if overrides is not None:
        config.update(overrides)
    
    writer = XamlWriter(flowdocument=flowdocument, xclass=xclass)
    rv = publish_string(source=input_data, writer=writer,
                        settings_overrides=config)
    return rv.root.to_string()


import unittest

from textwrap import dedent

from xamlwriter.writer import publish_xaml

# This installs the pygments directive
import xamlwriter.register_directive


def make_source(string):
    return ('.. code-block:: python\n\n    ' +
            '\n    '.join(string.splitlines()))

def make_doc(string):
    return ('<FlowDocument FontSize="15" xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation" '
            'xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml">'
            '<Paragraph FontFamily="Consolas, Global Monospace" xml:space="preserve">%s'
            '</Paragraph></FlowDocument>' % string)

def make_doc_sl(string):
    return ('<StackPanel x:Class="System.Windows.Controls.StackPanel" '
            'xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation" '
            'xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml">'
            '<TextBlock FontFamily="Consolas, Global Monospace" FontSize="15" Margin="0,10,0,0">'
            '%s</TextBlock></StackPanel>') % string

class TestPygments(unittest.TestCase):
    
    def testComment(self):
        # smoke test
        source = make_source('# comment')
        
        expected = make_doc('<Run Foreground="#408080" FontStyle="Italic"># comment</Run><Run></Run>\n')
        self.assertEqual(publish_xaml(source), expected)
    

    def testXamlEscape(self):
        source = make_source('"&<\'>"')
        expected = make_doc('<Run Foreground="#BA2121">&quot;&amp;&lt;&apos;&gt;&quot;</Run><Run></Run>\n')
        self.assertEqual(publish_xaml(source), expected)

        
    def testSilverlightWhitespaceHandling(self):
        xamlwriter.register_directive.flowdocument = False
        try:
            source = make_source('"foo  foo"\n"foo  foo"')
            expected = make_doc_sl('<Run Foreground="#BA2121">'
                                '&quot;foo&#0160;&#0160;foo&quot;</Run>'
                                '<Run></Run><LineBreak />'
                                '<Run Foreground="#BA2121">'
                                '&quot;foo&#0160;&#0160;foo&quot;</Run>'
                                '<Run></Run><LineBreak />')
            self.assertEqual(publish_xaml(source, flowdocument=False), expected)
        finally:
            xamlwriter.register_directive.flowdocument = True


    def DONTtestClass(self):
        source = make_source("""\
        import foo
        
        assert something
        
        class Foo(object):
            def method(self, arg1, arg2=None):
                if a == b:
                    return 3
                
                raise Exception("Weird error")
        """)
        
        print publish_xaml(source)



if __name__ == '__main__':
    unittest.main()



import copy
import re

from docutils import nodes
from docutils.nodes import NodeVisitor, SkipNode

from xamlwriter.node import ErrorNode, Node, TextNode
from xamlwriter.utils import escape_xaml


FONT_SIZE = '15'
MARGIN = "16,0,0,0"

class XamlTranslator(NodeVisitor):

    def __init__(self, document, flowdocument=True, xclass=True):
        NodeVisitor.__init__(self, document)
        self.flowdocument = flowdocument
        if flowdocument:
            self.root = Node('FlowDocument')
            self.root.attributes['FontSize'] = FONT_SIZE
        else:
            self.root = Node('StackPanel')
            if xclass:
                self.root.attributes['x:Class'] = 'System.Windows.Controls.StackPanel'
        self.root.attributes['xmlns'] = "http://schemas.microsoft.com/winfx/2006/xaml/presentation"
        self.root.attributes['xmlns:x'] = "http://schemas.microsoft.com/winfx/2006/xaml"
        self.curnode = self.root
        self.context = []
        self.initial_header_level = 2
        self.section_level = 0
        self.in_literal = False
        self.list_item = 0
        self.done_first_item = True
        self.bullet_list = True

    def begin_node(self, node, tagname, **more_attributes):
        new_node = Node(tagname)
        attributes = new_node.attributes
        for name, value in more_attributes.iteritems():
            attributes[name] = value
        if node is not None:
            # ids and classes handled here
            pass
        new_node.parent = self.curnode
        self.curnode.children.append(new_node)
        self.curnode = new_node

    def end_node(self):
        self.curnode = self.curnode.parent

    def add_text(self, text, escape=True):
        if not text:
            return
        if escape:
            text = escape_xaml(text)
        if not self.in_literal:
            self.curnode.children.append(TextNode(text))
            return
        assert not self.flowdocument
        text = text.replace(' ', '&#160;')
        parts = text.split('\n')
        for part in parts[:-1]:
            self.curnode.children.append(TextNode(part))
            self.curnode.children.append(Node('LineBreak'))
        self.curnode.children.append(TextNode(parts[-1]))

    def add_node(self, name, text='', escape=True, **attributes):
        self.begin_node(None, name, **attributes)
        self.add_text(text, escape=escape)
        self.end_node()

    def unknown_visit(self, node):
        # should raise here or indicate unsupported feature some way
        return

    def unknown_departure(self, node):
        return

    trivial_nodes_flowdocument = {
        'paragraph': ('Paragraph', {}),
        'emphasis': ('Italic', {}),
        'strong': ('Bold', {}),
        'block_quote': ('Section', {'Margin': MARGIN}),
        'literal_block': ('Paragraph', {'FontFamily': 'Consolas, Global Monospace', 
                                        'xml:space': 'preserve'}),
        'superscript': ('Run', {'Typography.Variants': 'Superscript'}),
        'line_block': ('Paragraph', {}), 
        'bullet_list': ('List', {}),
        'list_item': ('ListItem', {}),
        'enumerated_list': ('List', {'MarkerStyle': 'Decimal'}),
        'literal': ('Run', {'FontFamily': 'Consolas, Global Monospace', 
                            'xml:space': 'preserve'})
    }
    
    trivial_nodes_silverlight = {
        'emphasis': ('Run', {'FontStyle': 'Italic'}),
        'strong': ('Run', {'FontWeight': 'Bold'}),
        'block_quote': ('StackPanel', {'Margin': MARGIN}),
    }
    
    @property
    def trivial_nodes(self):
        if self.flowdocument:
            return self.trivial_nodes_flowdocument
        return self.trivial_nodes_silverlight
        

    def dispatch_visit(self, node):
        # don't call visitor methods for trivial nodes
        node_name = node.__class__.__name__
        if node_name == 'Text':
            self.add_text(node.astext())
            raise SkipNode
            
        tagname, atts = self.trivial_nodes.get(node_name, (None, None))
        if tagname:
            self.begin_node(node, tagname, **atts)
        else:
            getattr(self, 'visit_' + node_name, self.unknown_visit)(node)

    def dispatch_departure(self, node):
        node_name = node.__class__.__name__
        
        tagname, atts = self.trivial_nodes.get(node_name, (None, None))
        if tagname:
            self.end_node()
        else:
            getattr(self, 'depart_' + node_name, self.unknown_departure)(node)

    def visit_raw(self, node):
        if 'xaml' in node.get('format', '').split():
            self.curnode.children.append(TextNode(node.astext()))
        raise SkipNode
    
    def visit_paragraph(self, node):
        # Silverlight only
        attrs = {'FontSize': FONT_SIZE, 'TextWrapping': "Wrap"}
        if self.done_first_item:
            attrs['Margin'] = "0,10,0,0"
        self.begin_node(node, 'TextBlock', **attrs)
        self.done_first_item = True
        
    def depart_paragraph(self, node):
        self.end_node()
    
    def visit_line_block(self, node):
        # Silverlight only
        self.begin_node(node, 'TextBlock', Margin= "0,10,0,0", FontSize=FONT_SIZE, TextWrapping="Wrap")
        
    def depart_line_block(self, node):
        if self.curnode.children[-1] == Node('LineBreak'):
            self.curnode.children.pop()

        self.end_node()
        
    def visit_line(self, node):
        pass
    
    def depart_line(self, node):
        self.add_node('LineBreak')
    
    def visit_title(self, node):
        begun = False
        node_type = 'Paragraph'
        attrs = {}
        if not self.flowdocument:
            node_type = 'TextBlock'
            attrs['Margin'] = '0,10,0,0'
        if isinstance(node.parent, nodes.document):
            begun = True
            self.begin_node(node, node_type, FontSize='20', FontWeight='Bold', **attrs)
        elif isinstance(node.parent, nodes.section):
            if (len(node.parent) >= 2 and
                isinstance(node.parent[1], nodes.subtitle)):
                attrs['FontStyle'] = 'Italic'
            attrs['FontSize'] = str(20 - self.section_level)
            self.begin_node(node, node_type, **attrs)
            begun = True
            # We don't do back-reference link for title
        else:
            # also used for sidebar, topic, 
            # Admonition, table (caption)
            pass
        self.context.append(begun)

    def depart_title(self, node):
        if self.context.pop():
            self.end_node()
    
    def visit_section(self, node):
        self.section_level += 1

    def depart_section(self, node):
        self.section_level -= 1
        
    def visit_subtitle(self, node):
        begun = False
        if isinstance(node.parent, nodes.document):
            node_type = 'Paragraph'
            attrs = {}
            if not self.flowdocument:
                node_type = 'TextBlock'
                attrs['Margin'] = '0,10,0,0'
            self.begin_node(node, node_type, FontSize='19',
                            FontStyle='Italic', **attrs)
            begun = True
        else:
            # Also used for sidebar and section
            pass
        self.context.append(begun)

    def depart_subtitle(self, node):
        if self.context.pop():
            self.end_node()
    
    def visit_literal_block(self, node):
        # only used for Silverlight
        self.in_literal = True
        self.begin_node(node, 'TextBlock', Margin="0,10,0,0", FontSize="15",
                        TextWrapping="Wrap", FontFamily="Consolas, Global Monospace")
        
    def depart_literal_block(self, node):
        self.in_literal = False
        self.end_node()
    
    def visit_enumerated_list(self, node):
        # only used for Silverlight
        self.bullet_list = False
        self.visit_bullet_list(node)
        
    def depart_enumerated_list(self, node):
        self.depart_bullet_list(node)
        
    def visit_bullet_list(self, node):
        # only used for Silverlight
        self.list_item = 0
        self.begin_node(node, 'Grid', Margin='0,10,0,0')
        self.begin_node(node, 'Grid.ColumnDefinitions')
        self.add_node('ColumnDefinition', Width='Auto')
        self.add_node('ColumnDefinition')
        self.end_node()

    def depart_bullet_list(self, node): 
        rows = Node('Grid.RowDefinitions')
        rows.children = [Node('RowDefinition') for _ in range(self.list_item)]
        self.curnode.children.insert(1, rows)
        self.end_node()
        self.bullet_list = True
        
    def visit_list_item(self, node):
        # only used for Silverlight
        self.done_first_item = False
        point = '&#8226;'
        if not self.bullet_list:
            point = str(self.list_item + 1) + '.'
        self.add_node('TextBlock', point, escape=False, 
                      **{'Grid.Column': '0', 'Grid.Row': str(self.list_item)})
        self.begin_node(node, 'StackPanel', Margin="5,0,0,5",
                      **{'Grid.Column': '1', 'Grid.Row': str(self.list_item)})
        self.list_item += 1

    def depart_list_item(self, node):
        self.done_first_item = True
        self.end_node()
    
    def visit_literal(self, node):
        # only used for Silverlight
        self.in_literal = True
        self.begin_node(node, 'Run', FontFamily='Consolas, Global Monospace')
    
    def depart_literal(self, node):
        self.in_literal = False
        self.end_node()



"""
Can use Floater for sidebar (FlowDocument).
"""


# requires IronPython and .NET 3

import sys
import clr
clr.AddReference('PresentationFramework')
clr.AddReference('PresentationCore')
clr.AddReference('windowsbase')
from System.Windows.Controls import *
from System.Windows.Markup import XamlReader
from System.Windows import Window, Application

if len(sys.argv) == 1:
    from Microsoft.Win32 import OpenFileDialog
    
    dialog = OpenFileDialog()
    dialog.ShowDialog()
    
    filename = dialog.FileName
    if filename is None:
        sys.exit()
    stream = dialog.OpenFile()
elif len(sys.argv) > 2:
    print 'display_xaml [xaml_file]'
    sys.exit(1)
else:
    from System.IO import File
    filename = sys.argv[1]
    stream = File.OpenRead(filename)

reader = FlowDocumentReader()
flowDocument = XamlReader.Load(stream)
stream.Close()
reader.Document = flowDocument
w = Window()
w.Title = "Displaying %r" % filename
w.Content = reader
Application().Run(w)





from pygments import highlight
from pygments.lexers import get_lexer_by_name, TextLexer

from docutils import nodes
from docutils.parsers.rst import directives

from xamlwriter.xamlformatter import XamlFormatter

# set externally to configure whether FlowDocument or Silverlight
# XAML is to be output
flowdocument = True

def pygments_directive(name, arguments, options, content, lineno,
                       content_offset, block_text, state, state_machine):
    try:
        lexer = get_lexer_by_name(arguments[0])
    except ValueError:
        # no lexer found - use the text one instead of an exception
        lexer = TextLexer()
    # take an arbitrary option if more than one is given
    formatter = XamlFormatter(flowdocument=flowdocument)
    parsed = highlight(u'\n'.join(content), lexer, formatter)
    return [nodes.raw('', parsed, format='xaml')]

pygments_directive.arguments = (1, 0, 1)
pygments_directive.content = 1
pygments_directive.options = {}

directives.register_directive('code-block', pygments_directive)



import unittest

from xamlwriter.node import ErrorNode, Node, OrderedDict, TextNode


class TestNode(unittest.TestCase):
    
    def testConstruction(self):
        node = Node('foo')
        self.assertEqual(node.name, 'foo')
        self.assertEqual(node.attributes, OrderedDict())
        self.assertEqual(node.children, [])
        self.assertEqual(node.parent, None)


    def testEquality(self):
        node = Node('foo')
        self.assertEqual(node, Node('foo'))
        self.assertNotEqual(node, Node('bar'))
        
        node.attributes['foo'] = 'fish'
        node2 = Node('foo')
        self.assertNotEqual(node, node2)
        
        node2.attributes['foo'] = 'fish'
        self.assertEqual(node, node2)
        
        node.children.append(Node('bar'))
        self.assertNotEqual(node, node2)
        
        node2.children.append(Node('bar'))
        self.assertEqual(node, node2)


    def testHash(self):
        self.assertRaises(TypeError, lambda: hash(Node('foo')))


    def testComparison(self):
        node = Node('foo')
        for name in '__gt__ = __lt__ = __ge__ = __le__'.split(' = '):
            method = getattr(node, name)
            self.assertTrue(method(None) is NotImplemented)


    def testToStringSimple(self):
        node = Node('foo')
        self.assertEqual(node.to_string(), '<foo />')


    def testToStringChildren(self):
        node = Node('foo')
        node.children.append(Node('bar'))
        self.assertEqual(node.to_string(), '<foo><bar /></foo>')


    def testToStringAttributes(self):
        node = Node('foo')
        node.attributes['bar'] = 'baz'
        self.assertEqual(node.to_string(), '<foo bar="baz" />')


class TestTextNode(unittest.TestCase):

    def testConstruction(self):
        node = TextNode('some text')
        self.assertEqual(node.data, 'some text')
        
        self.assertFalse(hasattr(node, 'name'))
        self.assertFalse(hasattr(node, 'attributes'))
        self.assertFalse(hasattr(node, 'children'))


    def testEquality(self):
        node = TextNode('some text')
        self.assertEqual(node, TextNode('some text'))
        self.assertNotEqual(node, TextNode('other text'))


    def testHash(self):
        self.assertRaises(TypeError, lambda: hash(TextNode('foo')))


    def testComparison(self):
        node = TextNode('foo')
        for name in '__gt__ = __lt__ = __ge__ = __le__'.split(' = '):
            method = getattr(node, name)
            self.assertTrue(method(None) is NotImplemented)


    def testToString(self):
        node = TextNode('foo')
        self.assertEqual(node.to_string(), 'foo')


class TestErrorNode(unittest.TestCase):
    
    def testErrorNode(self):
        self.assertTrue(issubclass(ErrorNode, Node))



if __name__ == '__main__':
    unittest.main()




# a xaml formatter for pygments

from pygments.formatter import Formatter

from xamlwriter.utils import escape_xaml

__all__ = ['XamlFormatter']


class XamlFormatter(Formatter):
    name = 'XAML'
    aliases = ['xaml']
    filenames = ['*.xaml']

    def __init__(self, flowdocument=True, **options):
        Formatter.__init__(self, **options)
        self.flowdocument = flowdocument
        
        self.linenos = 0
        if flowdocument:
            self.lineseparator = '\n'
        else:
            self.lineseparator = '<LineBreak />'
        self.styles = {}
        
        for token, style in self.style:
            format_string = ''
            # a style item is a tuple in the following form:
            # colors are readily specified in hex: 'RRGGBB'
            if style['color']:
                format_string = ' Foreground="#%s"' % style['color']
            if style['bold']:
                format_string += ' FontWeight="Bold"'
            if style['italic']:
                format_string += ' FontStyle="Italic"'
            if style['underline']:
                # not used ?
                pass
            self.styles[token] = format_string


    def format(self, tokensource, outfile):
        """
        The formatting process uses several nested generators; which of
        them are used is determined by the user's options.

        Each generator should take at least one argument, ``inner``,
        and wrap the pieces of text generated by this.

        Always yield 2-tuples: (code, text). If "code" is 1, the text
        is part of the original tokensource being highlighted, if it's
        0, the text is some piece of wrapping. This makes it possible to
        use several different wrappers that process the original source
        linewise, e.g. line number generators.
        """
        source = self._format_lines(tokensource)
        
        if self.flowdocument:
            start = '<Paragraph FontFamily="Consolas, Global Monospace" xml:space="preserve">'
            end = '</Paragraph>'
        else:
            start = '<TextBlock FontFamily="Consolas, Global Monospace" FontSize="15" Margin="0,10,0,0">'
            end = '</TextBlock>'
            
        outfile.write(start)
        for t, piece in source:
            outfile.write(piece)
        
        outfile.write(end)
            

    def _format_lines(self, tokensource):
        """
        Just format the tokens, without any wrapping tags.
        Yield individual lines.
        """
        enc = self.encoding
        lsep = self.lineseparator
        

        lspan = ''
        line = ''
        for ttype, value in tokensource:
            cspan = '<Run%s>' % self.styles[ttype]
            
            value = escape_xaml(value)
            if not self.flowdocument:
                value = value.replace(' ', '&#0160;')
            parts = value.split('\n')
            
            # for all but the last line
            for part in parts[:-1]:
                if line:
                    if lspan != cspan:
                        line += (lspan and '</Run>') + cspan + part + \
                                (cspan and '</Run>') + lsep
                    else: # both are the same
                        line += part + (lspan and '</Run>') + lsep
                    yield 1, line
                    line = ''
                elif part:
                    yield 1, cspan + part + (cspan and '</Run>') + lsep
                else:
                    yield 1, lsep
            # for the last line
            if line and parts[-1]:
                if lspan != cspan:
                    line += (lspan and '</Run>') + cspan + parts[-1]
                    lspan = cspan
                else:
                    line += parts[-1]
            elif parts[-1]:
                line = cspan + parts[-1]
                lspan = cspan
            # else we neither have to open a new span nor set lspan

        if line:
            yield 1, line + (lspan and '</Run>') + lsep
            





# Using XAML escape rules (XML) from:
# http://msdn.microsoft.com/en-us/library/ms748250.aspx

def escape_xaml(text):
    """Escape &, <, > as well as single and double quotes for XML."""
    return text.replace('&', '&amp;').  \
                replace('<', '&lt;').   \
                replace('>', '&gt;').   \
                replace('"', '&quot;'). \
                replace("'", '&apos;')



#!/usr/bin/env python

import sys

# This installs the pygments directive
import xamlwriter.register_directive

from xamlwriter.writer import publish_xaml


USAGE = "rst2xaml input_file output_file"

if len(sys.argv) != 3:
    print USAGE
    sys.exit(1)
    
input_data = open(sys.argv[1]).read().decode('utf-8')
#print input_data

output = publish_xaml(input_data)
#print output

handle = open(sys.argv[2], 'w')
handle.write(output.encode('utf-8'))
handle.close()


from distutils.core import setup
from xamlwriter import __version__ as version

description = """A docutils writer for FlowDocument XAML.

Includes an ``rst2xaml`` script that takes a reStructured Text source document
and outputs a XAML document.

FlowDocument XAML is a text markup for displaying documents in the Microsoft
Windows Presentation Foundation (WPF) user interface - which is also used in
the Silverlight / Moonlight browser plugins.

The goal is to be able to write Python tutorials in reST and present them on the
web with a Python interactive interpreter running in the browser. This will also
require XAML output from Pygments."""


setup(
    name='rst2xaml',
    version=version,
    description=description,
    author="Michael Foord",
    author_email="michael@voidspace.org.uk",
    packages= ['xamlwriter', 'xamlwriter.tests', 'xamlwriter.modules'],
    scripts=['rst2xaml.py', 'rst2xamlsl.py', 'display_xaml.py']
)



import unittest
from textwrap import dedent
from docutils.core import publish_string

from xamlwriter.node import Node, TextNode
from xamlwriter.translator import FONT_SIZE, MARGIN
from xamlwriter.writer import XamlWriter, publish_xaml


settings_overrides = {
    'file_insertion_enabled': False,
}

def tree_from_string(input_data):
    input_data = dedent(input_data)
    rv = publish_string(source=input_data, writer=XamlWriter(),
                        settings_overrides=settings_overrides)

    return rv.root


def get_root():
    node = Node('FlowDocument')
    node.attributes['FontSize'] = FONT_SIZE
    node.attributes['xmlns'] = "http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    node.attributes['xmlns:x'] = "http://schemas.microsoft.com/winfx/2006/xaml"
    return node


def tree_from_string_sl(input_data):
    input_data = dedent(input_data)
    writer = XamlWriter(flowdocument=False)
    rv = publish_string(source=input_data, writer=writer,
                        settings_overrides=settings_overrides)

    return rv.root


def get_root_sl():
    node = Node('StackPanel')
    node.attributes['x:Class'] = 'System.Windows.Controls.StackPanel'
    node.attributes['xmlns'] = "http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    node.attributes['xmlns:x'] = "http://schemas.microsoft.com/winfx/2006/xaml"
    return node


def get_sl_paragraph():
    para = Node('TextBlock')
    para.attributes['FontSize'] = FONT_SIZE
    para.attributes['Margin'] = "0,10,0,0"
    para.attributes['TextWrapping'] = "Wrap"
    return para

def get_grid_sl(items, enumerated=True):
    node = get_root_sl()
    grid = Node('Grid')
    grid.attributes['Margin'] = '0,10,0,0'
    node.children.append(grid)
    columns = Node('Grid.ColumnDefinitions')
    grid.children.append(columns)
    col = Node('ColumnDefinition')
    col.attributes['Width'] = 'Auto'
    columns.children.append(col)
    columns.children.append(Node('ColumnDefinition'))
    rows = Node('Grid.RowDefinitions')
    grid.children.append(rows)
    rows.children = [Node('RowDefinition') for _ in items]
    
    for i, item in enumerate(items):
        bullet = Node('TextBlock')
        point = '&#8226;'
        if enumerated:
            point = str(i + 1) + '.'
        bullet.attributes['Grid.Row'] = str(i)
        bullet.attributes['Grid.Column'] = '0'
        bullet.children.append(TextNode(point))
                               
        bullet_text = Node('StackPanel')
        bullet_text.attributes['Margin'] = "5,0,0,5"
        bullet_text.attributes['Grid.Row'] = str(i)
        bullet_text.attributes['Grid.Column'] = '1'
        para = get_sl_paragraph()
        del para.attributes['Margin']
        para.children.append(TextNode(item))
        bullet_text.children.append(para)
        
        grid.children.append(bullet)
        grid.children.append(bullet_text)
    return node



class TestXamlWriter(unittest.TestCase):

    def testBasic(self):
        tree = tree_from_string('')
        self.assertEqual(tree, get_root())
    
        
    def testParagraph(self):
        tree = tree_from_string('Hello')
        
        node = get_root()
        node.children.append(Node('Paragraph'))
        node.children[0].children.append(TextNode('Hello'))
        self.assertEqual(tree, node)

        
    def testXamlEscape(self):
        tree = tree_from_string('"&<\'>"')
        
        node = get_root()
        node.children.append(Node('Paragraph'))
        node.children[0].children.append(TextNode('&quot;&amp;&lt;&apos;&gt;&quot;'))
        self.assertEqual(tree, node)
    
        
    def testItalics(self):
        tree = tree_from_string('*Hello*')
        
        node = get_root()
        para = Node('Paragraph')
        para.children.append(Node('Italic'))
        para.children[0].children.append(TextNode('Hello'))
        node.children.append(para)
        self.assertEqual(tree, node)
    
        
    def testBold(self):
        tree = tree_from_string('**Hello**')
        
        node = get_root()
        para = Node('Paragraph')
        para.children.append(Node('Bold'))
        para.children[0].children.append(TextNode('Hello'))
        node.children.append(para)
        self.assertEqual(tree, node)
    
        
    def testBlockquote(self):
        tree = tree_from_string('Hello\n\n    foo\n')
        
        node = get_root()
        para = Node('Paragraph')
        para.children.append(TextNode('Hello'))
        node.children.append(para)
        
        section = Node('Section')
        section.attributes['Margin'] = "16,0,0,0"
        quote = Node('Paragraph')
        quote.children.append(TextNode('foo'))
        section.children.append(quote)
        
        node.children.append(section)
        
        self.assertEqual(tree, node)


    def testRawXaml(self):
        data = '.. raw:: xaml\n\n   foo'
        output = publish_xaml(data)
        node = get_root()
        node.children.append(TextNode('foo'))
        self.assertEqual(output, node.to_string())
        
    
    def testLiteralBlock(self):
        node = get_root()
        literal = Node('Paragraph')
        literal.attributes['FontFamily'] = 'Consolas, Global Monospace'
        literal.attributes['xml:space'] = 'preserve'
        node.children.append(literal)
        literal.children.append(TextNode('foo'))
        
        self.assertEqual(tree_from_string('::\n\n    foo'), node)
    
       
    def testSuperscript(self):
        node = get_root()
        para = Node('Paragraph')
        superscript = Node('Run')
        superscript.attributes['Typography.Variants'] = 'Superscript'
        para.children.append(superscript)
        node.children.append(para)
        superscript.children.append(TextNode('foo'))
        
        self.assertEqual(tree_from_string(':sup:`foo`'), node)
    
    
    def testLineBlock(self):
        node = get_root()
        para = Node('Paragraph')
        para.children.append(TextNode('foo'))
        para.children.append(Node('LineBreak'))
        para.children.append(TextNode('bar'))
        para.children.append(Node('LineBreak'))
        
        node.children.append(para)
        
        actual = tree_from_string("""\
            | foo
            | bar""")
        
        self.assertEqual(actual, node)
    
    
    def testBulletList(self):
        node = get_root()
        def get_item(text):
            node = Node('ListItem')
            para = Node('Paragraph')
            para.children.append(TextNode(text))
            node.children.append(para)
            return node
        
        list_node = Node('List')
        list_node.children.append(get_item('first'))
        list_node.children.append(get_item('second'))
        list_node.children.append(get_item('third'))
        
        node.children.append(list_node)
        
        actual = tree_from_string("""\
            * first
            * second
            * third""")
        
        self.assertEqual(actual, node)
    
    
    def testEnumeratedList(self):
        node = get_root()
        def get_item(text):
            node = Node('ListItem')
            para = Node('Paragraph')
            para.children.append(TextNode(text))
            node.children.append(para)
            return node
        
        list_node = Node('List')
        list_node.attributes['MarkerStyle'] = 'Decimal'
        list_node.children.append(get_item('first'))
        list_node.children.append(get_item('second'))
        list_node.children.append(get_item('third'))
        
        node.children.append(list_node)
        
        actual = tree_from_string("""\
            1. first
            2. second
            3. third""")
        
        self.assertEqual(actual, node)
        
    
    def testLiteral(self):
        node = get_root()
        para = Node('Paragraph')
        literal = Node('Run')
        literal.attributes['FontFamily'] = 'Consolas, Global Monospace'
        literal.attributes['xml:space'] = 'preserve'
        para.children.append(literal)
        literal.children.append(TextNode('foo  bar'))
        node.children.append(para)
        
        self.assertEqual(tree_from_string('``foo  bar``'), node)

    def testTitleSubtitleSection(self):
        def get_title(fontsize, text, **attributes):
            node = Node('Paragraph')
            node.children.append(TextNode(text))
            node.attributes.update(attributes.items())
            node.attributes['FontSize'] = str(fontsize)
            return node
        
        node = get_root()
        node.children.append(get_title(20, 'Title', FontWeight='Bold'))
        node.children.append(get_title(19, 'Subtitle', FontStyle='Italic'))
        node.children.append(get_title(19, 'Heading 1'))
        node.children.append(get_title(18, 'Heading 2'))
        node.children.append(get_title(17, 'Heading 3'))
        node.children.append(get_title(16, 'Heading 4'))
        
        source = """\
        =======
         Title
        =======
        ----------
         Subtitle
        ----------
        
        Heading 1
        =========
        
        Heading 2
        ---------
        
        Heading 3
        ~~~~~~~~~
        
        Heading 4
        #########
        """
        result = tree_from_string(source)
        self.assertEqual(result, node)



class TestSilverlightXaml(unittest.TestCase):

    def testBasic(self):
        tree = tree_from_string_sl('')
        self.assertEqual(tree, get_root_sl())

        
    def testParagraph(self):
        tree = tree_from_string_sl('Hello')
        
        node = get_root_sl()
        para = get_sl_paragraph()
        node.children.append(para)
        node.children[0].children.append(TextNode('Hello'))
        self.assertEqual(tree, node)
    
        
    def testItalics(self):
        tree = tree_from_string_sl('*Hello*')
        
        node = get_root_sl()
        para = get_sl_paragraph()
        italic = Node('Run')
        italic.attributes['FontStyle'] = 'Italic'
        para.children.append(italic)
        para.children[0].children.append(TextNode('Hello'))
        
        node.children.append(para)
        self.assertEqual(tree, node)
    
        
    def testBold(self):
        tree = tree_from_string_sl('**Hello**')
        
        node = get_root_sl()
        para = get_sl_paragraph()
        italic = Node('Run')
        italic.attributes['FontWeight'] = 'Bold'
        para.children.append(italic)
        para.children[0].children.append(TextNode('Hello'))
        
        node.children.append(para)
        self.assertEqual(tree, node)


    def testBlockquote(self):
        tree = tree_from_string_sl('Hello\n\n    foo\n')
        
        node = get_root_sl()
        para = get_sl_paragraph()
        para.children.append(TextNode('Hello'))
        node.children.append(para)
        
        block = Node('StackPanel')
        block.attributes['Margin'] = MARGIN
        para2 = get_sl_paragraph()
        para2.children.append(TextNode('foo'))
        block.children.append(para2)
        
        node.children.append(block)
        
        self.assertEqual(tree, node)
        

    def testRawXaml(self):
        data = '.. raw:: xaml\n\n   foo'
        output = publish_xaml(data, flowdocument=False)

        node = get_root_sl()
        node.children.append(TextNode('foo'))
        self.assertEqual(output, node.to_string())
        
    
    def testLiteralBlock(self):
        node = get_root_sl()
        
        literal = get_sl_paragraph()
        literal.attributes['FontFamily'] = 'Consolas, Global Monospace'
        # XXXX do we want the literal paragraph (TextBlock) to have NoWrap set?
        node.children.append(literal)
        
        literal.children.append(TextNode('foo'))
        literal.children.append(Node('LineBreak'))
        literal.children.append(TextNode('foo&#160;&#160;foo'))
        
        result = tree_from_string_sl('::\n\n    foo\n    foo  foo\n')
        self.assertEqual(result, node)
    
    
    def testLineBlock(self):
        node = get_root_sl()
        para = get_sl_paragraph()
        para.children.append(TextNode('foo'))
        para.children.append(Node('LineBreak'))
        para.children.append(TextNode('bar'))
        
        node.children.append(para)
        
        actual = tree_from_string_sl("""\
            | foo
            | bar""")
        
        self.assertEqual(actual, node)
    
    
    def testBulletList(self):
        node = get_grid_sl(['first', 'second', 'third'], enumerated=False)
        actual = tree_from_string_sl("""\
            * first
            * second
            * third""")
        
        self.assertEqual(actual, node)
    
    
    def testEnmeratedList(self):
        node = get_grid_sl(['first', 'second', 'third'])
        actual = tree_from_string_sl("""\
            #. first
            #. second
            #. third""")
        
        self.assertEqual(actual, node)
        
    
    def testLiteral(self):
        node = get_root_sl()
        para = get_sl_paragraph()
        literal = Node('Run')
        literal.attributes['FontFamily'] = 'Consolas, Global Monospace'
        para.children.append(literal)
        literal.children.append(TextNode('foo&#160;&#160;bar'))
        node.children.append(para)
        
        self.assertEqual(tree_from_string_sl('``foo  bar``'), node)
        
        
    def testTitleSubtitleSection(self):
        def get_title(fontsize, text, FontWeight=None, FontStyle=None):
            node = Node('TextBlock')
            node.children.append(TextNode(text))
            if FontWeight is not None:
                node.attributes['FontWeight'] = FontWeight
            node.attributes['Margin'] = "0,10,0,0"
            if FontStyle is not None:
                node.attributes['FontStyle'] = FontStyle
            node.attributes['FontSize'] = str(fontsize)
            return node
        
        node = get_root_sl()
        node.children.append(get_title(20, 'Title', FontWeight='Bold'))
        node.children.append(get_title(19, 'Subtitle', FontStyle='Italic'))
        node.children.append(get_title(19, 'Heading 1'))
        node.children.append(get_title(18, 'Heading 2'))
        node.children.append(get_title(17, 'Heading 3'))
        node.children.append(get_title(16, 'Heading 4'))
        
        source = """\
        =======
         Title
        =======
        ----------
         Subtitle
        ----------
        
        Heading 1
        =========
        
        Heading 2
        ---------
        
        Heading 3
        ~~~~~~~~~
        
        Heading 4
        #########
        """
        result = tree_from_string_sl(source)
        self.assertEqual(result, node)




if __name__ == '__main__':
    unittest.main()
